//
//  FieldTableViewCell.swift
//  traccar
//
//  Created by iOS Developer on 2/6/22.
//

import UIKit

final class FieldTableViewCell: KNTableViewCell {
    
    // MARK: - Outlets
    @IBOutlet var fieldView: KNTextFieldView!
    
    @IBOutlet private weak var fieldViewLeadingConstraint: NSLayoutConstraint!
    @IBOutlet private weak var fieldViewTrailingConstraint: NSLayoutConstraint!
    
    // MARK: - Properties
    private var indexPath: IndexPath?
    private var field: KNField?
    
    var isLeadingSpaceZero: Bool = false
    
    weak var delegate: KNTextFieldViewDelegate?
    weak var dropdownDelegate: KNTextFieldViewDropdownDelegate?
    weak var prefixDelegate: KNTextFieldViewPrefixDelegate?
    
    // MARK: - Lifecycle
    override func awakeFromNib() {
        super.awakeFromNib()
        
        setupUI()
    }
    
    // MARK: - Setup
    func set(field: KNField?, indexPath: IndexPath?) {
        self.indexPath = indexPath
        self.field = field
        fieldView.textField.textAlignment = .left
        
        updateUI()
    }
    
    func set(field: KNField?, indexPath: IndexPath?, isLeadingSpaceZero: Bool) {
        self.isLeadingSpaceZero = isLeadingSpaceZero
        self.set(field: field, indexPath: indexPath)
        fieldView.textField.textAlignment = .left
        if let field = field {
                fieldView.textField.textAlignment = .left
                fieldView.semanticContentAttribute = .forceLeftToRight
        }
    }
    
    // MARK: - UI
    private func setupUI() {
        contentView.backgroundColor = .white
    }
    
    private func updateUI() {
        guard let field = field else { return }
        
        fieldView.setupUI()

        fieldView.prefixDelegate = prefixDelegate
        fieldView.delegate = delegate
        fieldView.dropdownDelegate = dropdownDelegate
        fieldView.set(field: field)
        fieldView.textField.textAlignment = .left
        fieldView.semanticContentAttribute = .forceLeftToRight
      
        fieldViewLeadingConstraint.constant = isLeadingSpaceZero ? 0 : 16
        fieldViewTrailingConstraint.constant = isLeadingSpaceZero ? 0 : 16

    }
}
