//
//  KNTextFieldView.swift
//  traccar
//

import UIKit

final class KNTextFieldView: KNComponentView {
    
    // MARK: - Outlets
    @IBOutlet private weak var view: UIView!
    @IBOutlet private weak var boxView: UIView!
    
    // top
    @IBOutlet private weak var topView: UIView!
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var titleLabel_bottom: NSLayoutConstraint!
    @IBOutlet private weak var countLabel: UILabel!
    @IBOutlet private weak var countLabel_leading: NSLayoutConstraint!
    @IBOutlet private weak var countLabel_width: NSLayoutConstraint!
    
    // field
    @IBOutlet private weak var fieldView: UIView!
    @IBOutlet private weak var borderView: UIView!
    
    @IBOutlet private weak var prefixView: UIView!
    @IBOutlet private weak var prefixView_width: NSLayoutConstraint!
    @IBOutlet private weak var prefixFlagIcon: UIImageView!
    @IBOutlet private weak var prefixFlagIcon_leading: NSLayoutConstraint!
    @IBOutlet private weak var prefixFlagIcon_width: NSLayoutConstraint! //31
    @IBOutlet private weak var prefixFlagIcon_height: NSLayoutConstraint! //20
    @IBOutlet private weak var prefixLabel_leading: NSLayoutConstraint! //2
    @IBOutlet private weak var prefixLabel: UILabel!
    @IBOutlet private weak var prefixLabel_trailing: NSLayoutConstraint! //2
    @IBOutlet private weak var prefixArrowIcon: UIImageView!
    @IBOutlet private weak var prefixArrowIcon_width: NSLayoutConstraint! //8
    @IBOutlet private weak var prefixArrowIcon_height: NSLayoutConstraint! //4
    @IBOutlet private weak var prefixButton: UIButton!
    
    @IBOutlet private weak var textFieldView: UIView!
    @IBOutlet var textField: KNTextField!
    @IBOutlet private weak var textField_trailing: NSLayoutConstraint! //12
    @IBOutlet private weak var unitValueLabel: UILabel!
    
    @IBOutlet private weak var accessoryView: UIView!
    @IBOutlet private weak var accessoryViewLabel: UILabel!
    @IBOutlet private weak var accessoryViewIcon: UIImageView!
    @IBOutlet private weak var accessoryView_width: NSLayoutConstraint!
    @IBOutlet private weak var accessoryView_trailing: NSLayoutConstraint!
    @IBOutlet private weak var accessory_button: UIButton!
    @IBOutlet private weak var unitMeasurementDropdwonArrowImageView: UIImageView!
    @IBOutlet private weak var unitMeasurementDropdwonArrowImageView_width: NSLayoutConstraint!
    @IBOutlet private weak var unitMeasurementDropdwonArrowImageView_height: NSLayoutConstraint!
    
    // bottom
    @IBOutlet private weak var bottomView: UIView!
    @IBOutlet private weak var infoLabel: UILabel!
    @IBOutlet private weak var infoLabel_top: NSLayoutConstraint!
    
    @IBOutlet private weak var button: UIButton!
    // userName label
    @IBOutlet private weak var useNameLabel: UILabel!
    @IBOutlet private weak var userNameLabel_width: NSLayoutConstraint!
    @IBOutlet weak var userNameLabel_trailing: NSLayoutConstraint!
    @IBOutlet weak var userNameLabel_leading: NSLayoutConstraint!
    @IBOutlet weak var textField_leading: NSLayoutConstraint!
    
    // MARK: - Properties
    private var helper: KNTextFieldViewHelper?
    private var disposeBag = DisposeBag()
    
    var field: KNField?
    var unitValueField: KNTextFieldView?
    
    var isUnitMeasurement: (value: Bool, dropdown: Bool) = (value: false, dropdown: false)
    
    weak var delegate: KNTextFieldViewDelegate?
    weak var dropdownDelegate: KNTextFieldViewDropdownDelegate?
    weak var prefixDelegate: KNTextFieldViewPrefixDelegate?
    
    // MARK: - Lifecycle
    override func setupNib() {
        super.setupNib()
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        setupUI()
    }
    
    // MARK: - UI
    func setupUI() {
        backgroundColor = .clear
        view.backgroundColor = .clear
        boxView.backgroundColor = .clear
        topView.backgroundColor = .clear
        fieldView.backgroundColor = .clear
        borderView.backgroundColor = .white
        prefixView.backgroundColor = .clear
        textFieldView.backgroundColor = .clear
        accessoryView.backgroundColor = .clear
        textField.backgroundColor = .clear
        bottomView.backgroundColor = .clear
        
        titleLabel_bottom.set(0) //8
        prefixView_width.set(0) //100
        accessoryView_width.set(0) //50
        infoLabel_top.set(0) //8a
        
        useNameLabel.set(text: "")
        useNameLabel.hide()
        userNameLabel_width.set(0)
        userNameLabel_trailing.set(0)
        userNameLabel_leading.set(0)
        textField_leading.set(12)
        
        borderView.addBorder(radius: 13, width: 1, color: .main)
        
        titleLabel.set(text: "", color: .black, font: .semibold(12))
        titleLabel.numberOfLines = 1
        
        infoLabel.set(text: "", color: .black, font: .regular(12))
        infoLabel.numberOfLines = 0
        
        countLabel.set(text: "", color: .main, font: .medium(12))
        countLabel.numberOfLines = 1
        countLabel.align(to: .left)
        
        unitValueLabel.hide()
        
        textField.text = ""
        textField.placeholder = ""
        textField.delegate = self
        textField.isUserInteractionEnabled = true
        textField.tintColor = .main
        textField.textAlignment = .left
        
        accessoryViewLabel.set(text: "", color: .gray, font: .medium(16))
        accessoryViewLabel.numberOfLines = 1
        
        accessory_button.isUserInteractionEnabled = false
        accessory_button.addTarget(self, action: #selector(accessoryButtonTapped), for: .touchUpInside)
        
        accessoryView.hide()
        accessoryViewIcon.hide()
        
        button.addTarget(self, action: #selector(buttonTapped), for: .touchUpInside)
        button.isUserInteractionEnabled = false
        button.hide()
    }
    
    override func layoutSubviews() {
        super.layoutSubviews()
        //rtl
        //@TODO @RTL-ISSUE
        //everything is fine when we have layoustsubvies in textfield.
        // not working properly here. play around to find a solution
        //textField.textAlignment = lang == .en ? .right : .left
        //textField.semanticContentAttribute = lang == .en ? .forceRightToLeft : .forceLeftToRight
    }
    
    // MARK: - Setup
    func set(field: KNField?) {
        self.field = field
        textField.textAlignment = .left
        textField.semanticContentAttribute = .forceLeftToRight
        updateUI()
    }
    
    // MARK: - Handlers
    private func updateUI() {
        guard let field = field else { return }
        // helper
        
        helper = KNTextFieldViewHelper(field: field)
        guard let helper = helper else { return }
        
        //
        borderView.addBorder(radius: 13, width: 1, color: .main)
        
        textField.textAlignment = .left
        textField.semanticContentAttribute = .forceLeftToRight
        
        titleLabel.text = field.title == "" ? helper.title : field.title
        
        titleLabel_bottom.set(8)
        
        countLabel.text = ""
        countLabel.hide()
        
        // field
        textField.keyboardType = helper.keyboard
        textField.returnKeyType = helper.returnKey
        textField.isSecureTextEntry = helper.secureEntry
        
        if field.field == .iban {
            textField.autocapitalizationType = .allCharacters
        }
        
        if field.field == .price || field.field == .sale_price {
            textField.keyboardType = .decimalPad
        }
        
        textField.setForEnglighs(localizedPlaceholder: field.placeholder != "" ? field.placeholder : helper.placeholder, color: .lightGray, font: .regular(16))
        
        textField.show()
        unitValueLabel.hide()
        textField.text = field.value.value
        
        if !isUnitMeasurement.dropdown {
            unitValueLabel.hide()
            
            field.value.observe { [weak self] text in
                guard let strongSelf = self else { return }
                
                strongSelf.textField.text = text
                
                if text.count > 0 {
                    strongSelf.borderView.addBorder(radius: 13,
                                                    width: 2,
                                                    color: .main)
                    
                    if strongSelf.field?.field != .email || strongSelf.field?.field != .shipping_address_email || strongSelf.field?.field != .billing_address_email {
                        strongSelf.field?.error.value = ""
                    }
                } else {
                    //@TODO: check the reset thign again(no reload/updates?)
                    strongSelf.resetTextField()
                }
                
            }.disposed(by: disposeBag)
            
            textField.bind { [weak self] binded in
                guard let strongSelf = self else { return }
                
                var text = binded
                
                field.value.value = text
                
                if text.count > 0 {
                    strongSelf.borderView.addBorder(radius: 13, width: 2, color: .main)
                }
                
                //strongSelf.validate()
                
                if text.count == 0 {
                    //@TODO no reset necessary?
                    strongSelf.resetTextField()
                }
                
                strongSelf.delegate?.updateUI(for: field)
            }
            
            textField.endEditingBind { [weak self] endEditingBinded in
                guard let strongSelf = self else { return }
                strongSelf.validate()
                strongSelf.delegate?.updateUI(for: field)
            }
        }
        
        // accessory
        
        if field.field == .password {
            accessoryViewIcon.set(icon: helper.icon)
            accessoryView_width.set(50)
            accessoryView.show()
            accessoryViewIcon.show()
            accessory_button.show()
            accessory_button.isUserInteractionEnabled = true
        }
        
        // info
        if field.error.value.count > 0 {
            if isUnitMeasurement.dropdown { return }
            
            set(error: field.error.value)
            
        } else {
            
            infoLabel_top.set(0)
            infoLabel.text = ""
        }
        
        field.error.observe { [weak self] text in
            guard let strongSelf = self else { return }
            
            if strongSelf.isUnitMeasurement.dropdown { return }
            
            if text.count > 0 {
                strongSelf.borderView.addBorder(radius: 13, width: 2, color: .main)
                strongSelf.set(error: text)
                strongSelf.delegate?.updateUI(for: field)
            }
            
            if text.count == 0 {
                //@TODO: reset the field but without any update
                // test more for ERROR case
                strongSelf.resetTextField()
            }
        }.disposed(by: disposeBag)
        
        // ////////////////
        
        // extra settings
        if isUnitMeasurement.dropdown {
            textField.isUserInteractionEnabled = false
        }
        
        if field.editable {
            textField.isUserInteractionEnabled = true
        } else {
            textField.isUserInteractionEnabled = false
        }
        
        // update field
        if let text = textField.text {
            
            if isUnitMeasurement.dropdown { return }
            
            if text.count > 0 {
                borderView.addBorder(radius: 13, width: 2, color: .main)
            }
        }
        textField.textAlignment = .left
        textField.semanticContentAttribute = .forceLeftToRight
        
        
    }
    
    // MARK: - Handler
    @objc private func buttonTapped() {
        
    }
    
    @objc private func prefixTapped() {
        
    }
    
    @objc private func accessoryButtonTapped() {
        textField.isSecureTextEntry.toggle()
        accessoryViewIcon.set(icon: textField.isSecureTextEntry ? .field_show_password : .field_hide_password)
    }
    
    //@TODO @bibi add network error field
}

// MARK: - UI Helpers - Error
extension KNTextFieldView {
    private func validate() {
        guard let field = field else { return }
        
        if field.field == .email || field.field == .shipping_address_email || field.field ==  .billing_address_email{
            guard let text = textField.text else { return }
            
            if text.isValidEmail {
                field.error.value = ""
                //@TODO reset func
                //resetTextField()
            } else {
                field.error.value = helper?.error ?? ""
                set(error: helper?.error)
            }
        }
    }
    
    private func set(error: String?) {
        guard let error = error else { return }
        
        if isUnitMeasurement.dropdown { return }
        
        infoLabel_top.set(8)
        infoLabel.set(localized: error, color: .red, font: .regular(12))
        
        borderView.addBorder(radius: 13, width: 2, color: .red)
        textField.textAlignment = .left
    }
    
    private func resetTextField() {
        if isUnitMeasurement.dropdown { return }
        guard let field = field else { return }
        textField.textAlignment = .left
        textField.semanticContentAttribute = .forceLeftToRight
        //        textField.textAlignment = lang == .en ? .left : .right
        //        textField.semanticContentAttribute = lang == .en ? .forceLeftToRight : .forceRightToLeft
        //
        set(field: field)
    }
}

// MARK: - TextField Delegate
extension KNTextFieldView: UITextFieldDelegate {
    func textFieldShouldBeginEditing(_ textField: UITextField) -> Bool {
        guard let field = field else { return true }
        
        if field.field == .sale_start_date || field.field == .sale_end_date {
            return false
        }
        
        return true
    }
    
    func textField(_ textField: UITextField, shouldChangeCharactersIn range: NSRange, replacementString string: String) -> Bool {
        guard let field = field else { return true }
        
        if field.field == .password || field.field == .confirm_password || field.field == .old_password {
            if let textFieldString = textField.text {
                let nsString = NSString(string: textFieldString)
                let updatedString = nsString.replacingCharacters(in:range, with:string)
                textField.text = updatedString
                
                //Setting the cursor at the right place
                let selectedRange = NSMakeRange(range.location + string.count, 0)
                let from = textField.position(from: textField.beginningOfDocument, offset:selectedRange.location)
                let to = textField.position(from: from!, offset:selectedRange.length)
                textField.selectedTextRange = textField.textRange(from: from!, to: to!)
                
                //Sending an action
                textField.sendActions(for: .editingChanged)
                
                return false
            }
            return true
            
        }
        return true
    }
    
    func textFieldDidEndEditing(_ textField: UITextField) {
        
        //@TODO this is not ok as well
        //we could reset (but without the update(reload)
        resetTextField()
    }
}
