//
//  KNTextFieldViewHelper.swift
//  traccar
//
//  Created by iOS Developer on 2/7/22.
//

import Foundation
import UIKit
import Alamofire

final class KNTextFieldViewHelper {
    
    private var type: KNFieldType = .empty
    private var field: JSONField = .none
    
    // MARK: - Init
    init(field: KNField) {
        self.type = field.type
        self.field = field.field
    }
    
    // MARK: - UI
    let labelPadding: CGFloat = 8
    
    // MARK: - Localization
    var title: String {
        switch field {
        case .password:
            return "Password"
        case .domain:
            return "Domain"
        case .username:
            return "Username"
        default:
            return ""
        }
    }
    
    var placeholder: String {
        switch field {
        case .password:
            return "********"
        case .domain:
            return "Domain"
        case .username:
            return "Username"
        default:
            return ""
        }
    }
    
    var error: String {
        return ""
    }
    
    var icon: UIImage {
        
        if field == .password || field == .confirm_password || field == .old_password{
            return .field_show_password
        }
        
        return UIImage()
    }
    
    
    // keyboard type
    var keyboard: UIKeyboardType {
        
        let units: [JSONField] = [
            .weight, .width, .height, .length
        ]
        
        if units.contains(field) {
            return .decimalPad
        }
        
        switch field {
        case .phone, .shipping_address_phone, .billing_address_phone:
            return .phonePad
        case .email, .shipping_address_email, .billing_address_email:
            return .emailAddress
        case .block,
                .billing_address_block,
                .shipping_address_block,
                .avenue,
                .billing_address_avenue,
                .shipping_address_avenue,
                .floor,
                .billing_address_floor,
                .shipping_address_floor:
            return .numberPad
        default:
            return .default
        }
    }
    
    // field is password
    var secureEntry: Bool {
        switch field {
        case .password, .confirm_password, .old_password :
            return true
        default:
            return false
        }
    }
    
    // return key (next or done)
    var returnKey: UIReturnKeyType {
        switch field {
        case .phone:
            return .done
        default:
            return .next
        }
    }
}
