//
//  KNField.swift
// 
//
//  Created by iOS Developer on 2/7/22.
//

import Foundation
import UIKit

final class KNField {
    typealias KNFieldBuilderClosure = (KNFieldBuilder) -> Void
    
    // MARK: - Properties
    var type: KNFieldType = .empty
    var tag: KNFieldTag = .none
    var field: JSONField = .none
    var isRequired: Bool = false
    
    var value: Observable<String> = Observable("")
    var info: Observable<String> = Observable("")
    var error: Observable<String> = Observable("")
    
    var action: KNButtonViewAction = .tap
    var style: KNButtonViewStyle = .simple

    
    var title: String = ""
    var height: CGFloat = .automatic
    
    var hidden: Bool = false
        
    var enabled: Bool = true
    var editable: Bool = true
    
    var labelStyle: KNLableStyle = .simple
    var allignment: NSTextAlignment = .natural
    var placeholder: String = ""

    
    // MARK: - Init
    init() { }
    
    init(builder: KNFieldBuilder) {
        isRequired = builder.isRequired
        field = builder.field
        type = builder.type
        tag = builder.tag
        
        value.value = builder.value
        error.value = builder.error
        info.value = builder.info
        
        action = builder.action
        style = builder.style
        
        height = builder.height
        title = builder.title
        
        enabled = builder.enabled
        editable = builder.editable
        
        labelStyle = builder.labelStyle
        allignment = builder.allignment
        hidden = builder.hidden
        placeholder = builder.placeholder
    }
    
    convenience init(_ block: KNFieldBuilderClosure) {
        let builder = KNFieldBuilder()
        block(builder)

        self.init(builder: builder)
    }
    
    // MARK: - Helper
    
    // MARK: - JSON Param
    var param: PARAMS {
        var json: PARAMS = [:]
        
        guard field != .none else { return [:] }
        
        guard !hidden else { return [:] }
        
        switch type {
        case .text, .empty:
            if !value.value.isEmpty {
                json.add(key: field, value.value)
            }
        default:
            if !value.value.isEmpty {
                json.add(key: field, value.value)
            }
            
        }
        
        return json
    }
}
