//
//  AuthenticationHeaderView.swift
//  opa
//
//  Created by George Makhoul on 2/16/22.
//

import UIKit

protocol CustomHeaderViewDelegate {
    func rightButtonTapped(action: RightButtonAction)
    func leftButtonTapped(action: LeftButtonAction)
}

enum HeaderType {
    case home
    case settings
    case status
}

enum RightButtonAction {
    case goSettings
    case back
    case nothing
}

enum LeftButtonAction {
    case goStatus
    case apply
    case clear
    case nothing
}

final class CustomHeaderView: KNComponentView {
    
    // MARK: - Outlets
    @IBOutlet private weak var view: UIView!
    @IBOutlet private weak var boxView: UIView!
    @IBOutlet private weak var button: UIButton!
    @IBOutlet private weak var rightButton: UIButton!
    @IBOutlet private weak var backgroundImage: UIImageView!
    @IBOutlet private weak var titleLabel: UILabel!
    
    // MARK: - Properties
    private var delegate: CustomHeaderViewDelegate?
    private var headerType: HeaderType?
    private var buttonAction: RightButtonAction = .nothing
    private var leftButtonAction: LeftButtonAction = .nothing
    
    // MARK: - Lifecycle
    override func setupNib() {
        super.setupNib()
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        setupUI()
    }
    
    // MARK: - UI
    private func setupUI() {
        view.backgroundColor = .clear
        titleLabel.set(text: "", color: .main, font: .semibold(20))
        titleLabel.textAlignment = .center
        backgroundColor = .clear
        boxView.backgroundColor = .clear
        button.addTarget(self, action: #selector(buttonTapped), for: .touchUpInside)
        rightButton.addTarget(self, action: #selector(leftButtonTapped), for: .touchUpInside)
    }
    
    private func setHomeType() {
        self.buttonAction = .goSettings
        self.leftButtonAction = .goStatus
        
        button.set(title: "")
        rightButton.set(title: "")

        button.set(icon: .menu)
        rightButton.set(icon: .status)
    }
    
    private func setSettingsType() {
        self.buttonAction = .back
        self.leftButtonAction = .apply
        
        titleLabel.set(text: "Settings")
        titleLabel.textAlignment = .center
        
        button.set(title: "")
        rightButton.set(title: "")
        rightButton.isHidden = !UserDefaults.standard.bool(forKey: Keys.User.updateSettings)
        button.set(icon: .back)
        rightButton.set(icon: .apply)
    }
    
    private func setStatusType() {
        self.buttonAction = .back
        self.leftButtonAction = .clear
        
        titleLabel.set(text: "Transactions")
        titleLabel.textAlignment = .center
        
        button.set(title: "")
        rightButton.set(title: "")

        button.set(icon: .back)
        rightButton.set(icon: .bin)

    }
    
    func set(type: HeaderType, delegate: CustomHeaderViewDelegate) {
        switch type {
        case .home:
            setHomeType()
            
        case .settings:
            setSettingsType()

        case .status:
            setStatusType()
        }
        
        self.delegate = delegate
    }
    
    // MARK: - Methods
    @objc private func buttonTapped() {
        delegate?.leftButtonTapped(action: self.leftButtonAction)
    }
    
    @objc private func leftButtonTapped() {
        delegate?.rightButtonTapped(action: self.buttonAction)
    }
}
