//
//  ButtonTableViewCell.swift
//  opa
//
//  Created by iOS Developer on 2/7/22.
//

import UIKit

protocol ButtonTableViewCellDelegate: AnyObject {
    func cellButtonTapped(action: KNButtonViewAction)
}

final class ButtonTableViewCell: KNTableViewCell {
    
    // MARK: - Outlets
    @IBOutlet private weak var buttonView: KNButtonView!
    
    // MARK: - Properties
    private var indexPath: IndexPath?
    private var field: KNField?
    
    weak var delegate: ButtonTableViewCellDelegate?
    
    // MARK: - Lifecycle
    override func awakeFromNib() {
        super.awakeFromNib()
        
        setupUI()
    }
    
    private func setupUI() {
        contentView.backgroundColor = .clear
        layer.cornerRadius = 10
        self.contentView.layer.cornerRadius = 10
    }
    
    private func updateUI() {
        guard let field = field else { return }
        buttonView.layer.cornerRadius = 10
        buttonView.primary(title: field.title, style: field.style, action: field.action, delegate: self)
    }
    
    // MARK: - Setup
    func set(field: KNField?, indexPath: IndexPath?) {
        self.indexPath = indexPath
        self.field = field
        
        updateUI()
    }
}

// MARK: - Button Delegate
extension ButtonTableViewCell: KNButtonViewDelegate {
    func buttonTapped(with action: KNButtonViewAction) {
        delegate?.cellButtonTapped(action: action)
    }
}
