//
//  KNButtonView.swift
//  opa
//
//  Created by iOS Developer on 2/2/22.
//

import Foundation
import UIKit

final class KNButtonView: KNComponentView {
    
    // MARK: - Outlets
    @IBOutlet private weak var view: UIView!
    @IBOutlet private weak var boxView: UIView!
    @IBOutlet private weak var button: UIButton!
    
    // MARK: - Properties
    private var color: UIColor = .main
    private var title: String = ""
    private var cornerRadius: CGFloat = 10.0
    private var titleColor: UIColor = .white
    private var titleFont: UIFont = .semibold(17)
    private var icon: UIImage = UIImage()
    private var type: KNButtonViewType = .primary
    private var style: KNButtonViewStyle = .simple
    private var action: KNButtonViewAction = .tap
    
    weak var delegate: KNButtonViewDelegate?
    
    // MARK: - Lifecycle
    override func setupNib() {
        super.setupNib()
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        setupUI()
    }
    
    // MARK: - UI
    private func setupUI() {
        view.backgroundColor = .clear
        boxView.backgroundColor = .clear
        
        button.addTarget(self, action: #selector(buttonTapped), for: .touchUpInside)
    }
    
    private func adjustSpaceBetweenTitleAndIcon() {
        var padding: CGFloat = 5
        
        if style != .withIcon { return }
        
        
                button.titleEdgeInsets = UIEdgeInsets(top: 0, left: padding, bottom: 0, right: 0)
                button.imageEdgeInsets = UIEdgeInsets(top: 0, left: 0, bottom: 0, right: padding)
    }
    
    private func adjustIconPlacement() {
        if style != .withIconAfterText { return }
        
        let padding: CGFloat = 5
        
            button.semanticContentAttribute = .forceLeftToRight
            
            button.titleEdgeInsets = UIEdgeInsets(top: 0, left: padding, bottom: 0, right: 0)
            button.imageEdgeInsets = UIEdgeInsets(top: 0, left: 0, bottom: 0, right: padding)
        
        //@TODO @george clarify
        button.transform = CGAffineTransform(scaleX: -1.0, y: 1.0)
        button.titleLabel?.transform = CGAffineTransform(scaleX: -1.0, y: 1.0)
        button.imageView?.transform = CGAffineTransform(scaleX: -1.0, y: 1.0)
    }
    
    // MARK: - Logic
    @objc private func buttonTapped() {
        delegate?.buttonTapped(with: action)
    }
    
    func primary(color: UIColor, title: String, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.color = color
        self.title = title
        self.type = .primary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        updatePrimary()
    }
    
    func primary(title: String, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.type = .primary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        updatePrimary()
    }
    
    func primary(title: String, icon: UIImage, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.icon = icon
        self.type = .primary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        button.set(icon: icon)
        
        updatePrimary()
    }
    
    func primary(color: UIColor, cornerRadius: CGFloat, title: String, titleFont: UIFont, titleColor: UIColor, icon: UIImage, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.color = color
        self.cornerRadius = cornerRadius
        self.title = title
        self.titleFont = titleFont
        self.titleColor = titleColor
        self.icon = icon
        self.type = .primary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        button.set(icon: icon)
        
        updatePrimary()
    }
    
    func secondary(title: String, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.type = .secondary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        updateSecondary()
    }
    
    func secondary(title: String, icon: UIImage, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.icon = icon
        self.type = .secondary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        button.set(icon: icon)
        
        updateSecondary()
    }
    
    func tertiary(title: String, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.type = .tertiary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        updateTertiary()
    }
    
    func tertiary(title: String, icon: UIImage, style: KNButtonViewStyle, action: KNButtonViewAction, delegate: KNButtonViewDelegate?) {
        self.title = title
        self.icon = icon
        self.type = .tertiary
        self.style = style
        self.action = action
        self.delegate = delegate
        
        button.set(icon: icon)
        
        updateTertiary()
    }
    
    private func updatePrimary() {
        boxView.addCorners(cornerRadius)
        boxView.backgroundColor = style == .transparent ? .clear : color
        titleColor = style == .transparent ? color  : .white
        addBorder(radius: 10, width: style == .transparent ? 2 : 0, color: style == .transparent ? color  : .white)
        button.set(localized: title, color: titleColor, font: titleFont)
    }
    
    private func updateSecondary() {
        boxView.addCorners(10)
        
        if action == .retryPayment || action == .trackOrder {
            boxView.backgroundColor = .main
            
        } else {
            boxView.backgroundColor = .main
        }
        
        
        var color: UIColor = .white
        var font: UIFont = .semibold(17)
        
        if style == .transparent {
            boxView.backgroundColor = .main
            
            color = .main
            font = .semibold(14)
        }else if style == .transperantExtraBlue {
            boxView.backgroundColor = .main
            
            titleColor = .white
            titleFont = .semibold(14)
        }
        
        adjustSpaceBetweenTitleAndIcon()
        adjustIconPlacement()

        button.set(localized: title, color: color, font: font)
    }
    
    private func updateTertiary() {
        boxView.addCorners(10)
        boxView.backgroundColor = .main
        
        adjustSpaceBetweenTitleAndIcon()
        adjustIconPlacement()

        button.set(localized: title, color: .white, font: .semibold(17))
    }
    
    func setAllignment(_ side :UIControl.ContentHorizontalAlignment) {
        button.contentHorizontalAlignment = side
        boxView.addCorners(side == .center ? 10 : 0)
    }
    
    
    func secondaryGrey(title: String,
                       style: KNButtonViewStyle,
                       action: KNButtonViewAction,
                       delegate: KNButtonViewDelegate?) {
        self.title = title
        self.type = .secondaryGrey
        self.style = style
        self.action = action
        self.delegate = delegate
        
        boxView.addCorners(cornerRadius)
        boxView.backgroundColor = .main
        
        titleColor = .white
        titleFont = .semibold(17)
        
        button.set(localized: title, color: titleColor, font: titleFont)
    }
}
