//
//  Dictionary.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation

extension Dictionary where Key: ExpressibleByStringLiteral {
    // Custom enum keys
    subscript<Index: RawRepresentable>(index: Index) -> Value? where Index.RawValue == String {
        get {
            return self["\(index.rawValue.lowercased())" as! Key]
        }

        set {
            self["\(index.rawValue.lowercased())" as! Key] = newValue
        }
    }
}

extension Dictionary where Key == String, Value == Any {
    mutating func add(key: JSONField, _ value: Any?) {
        if let string = value as? String {
            self[key.rawValue] = string
        } else if let int = value as? Int {
            self[key.rawValue] = int
        } else if let double = value as? Double {
            self[key.rawValue] = double
        } else if let objects = value as? [JSON] {
            self[key.rawValue] = objects
        } else if let object = value as? JSON {
            self[key.rawValue] = object
        } else if value != nil {
            self[key.rawValue] = value
        }
    }
    
    mutating func add(_ k: String, _ value: Any?) {
        if let string = value as? String {
            self[k] = string
        } else if let int = value as? Int {
            self[k] = int
        } else if let double = value as? Double {
            self[k] = double
        } else if value != nil {
            self[k] = value
        }
    }
    
    mutating func add(_ params: PARAMS) {
        for (k, v) in params {
            add(k, v)
        }
    }
    
    mutating func add(jsons: [JSON]) {
        for json in jsons {
            for (k, v) in json {
                add(k, v)
            }
        }
    }
    
    func string(_ key: JSONField) -> String {
        if let str = self[key.rawValue] as? String {
            return str
        }
        
        return ""
    }
    
    func dictionary(_ key: JSONField) -> JSON {
        if let dict = self[key.rawValue] as? JSON {
            return dict
        }
        
        return [:]
    }
    
    func dictionaries(_ key: JSONField) -> [JSON]? {
        if let dicts = self[key.rawValue] as? [JSON] {
            return dicts
        }
        
        return nil
    }
    
    func bool(_ key: JSONField) -> Bool {
        if let bool = self[key.rawValue] as? Bool {
            return bool
        }
        
        return false
    }
    
    func int(_ key: JSONField) -> Int {
        if let int = self[key.rawValue] as? Int {
            return int
        }
        
        if let str = self[key.rawValue] as? String {
            if let int = Int(str) {
                return int
            }
            
            return 0
        }
        
        return 0
    }
}


