//
//  Globals.swift
//  TraccarClient
//
//  Created by George Makhoul on 29/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
import UIKit


// MARK: - Delay with Closure
func delay(_ by: Double = 2.0, completion: @escaping () -> ()) {
    DispatchQueue.main.asyncAfter(deadline: .now() + by) {
        completion()
    }
}

// MARK: - Main Thread
func main(_ completion: @escaping () -> ()) {
    DispatchQueue.main.async() {
        completion()
    }
}

var fleetUrl: String {
    switch app.environment {
    case .development, .sandbox:
        return "https://dev-fleet-api.nmo.ai/"
    case .production:
        return "https://fleet-api.nmo.ai/"
    }
}

var leaderServerURL: String {
    switch app.environment {
    case .development, .sandbox:
        return "https://dev-fleet-app.nmo.ai/"
    case .production:
        return "https://fleet-app.nmo.ai/"
    }
}

var showLeaderButton: Bool = false

// MARK: - Global Custom AlertView
typealias AlertViewClosure = () -> ()

func getDayName() -> String {
    let date = Date() // Use current date
    let dateFormatter = DateFormatter()
    dateFormatter.dateFormat = "EEEE"
    let dayOfWeekString = dateFormatter.string(from: date)
    return dayOfWeekString
}

// use this when you only need to display the error without any actions
func ok(_ message: String, viewController:KNViewController? = nil) {
    let vc = KNAlertViewController(type: .error, message: message)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    if let viewController = viewController {
        viewController.present(vc, animated: false, completion: nil)
    }else{
        topViewController()?.present(vc, animated: false, completion: nil)
    }
}

func ok(_ message: String, _ completion: AlertViewClosure?) {
    let vc = KNAlertViewController(type: .error, message: message)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    vc.confirmClosure = completion
    
    topViewController()?.present(vc, animated: false, completion: nil)
}

func ok(_ result: NetworkResultModel) {
    guard result.haveFieldsErrors == false else { return }
    log("error", result.displayError)
    var message: String = ""
    message = result.displayError
    
    var codeWithMessage = ""
    if app.environment != .production {
        codeWithMessage = "[\(result.statusCode)]\n\n" + message
    } else {
        codeWithMessage = message
    }
    
    let vc = KNAlertViewController(type: .error, message: codeWithMessage)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    
    topViewController()?.present(vc, animated: false, completion: nil)
}

// use this when you need to have the action from the button
func ok(_ result: NetworkResultModel, _ completion: AlertViewClosure?){
    guard result.displayError.isEmpty else { return }
    
    var message: String = ""
    
    if result.displayError.isEmpty {
        message = ""
    } else {
        message = result.displayError
    }
    
    var codeWithMessage = ""
    if app.environment != .production {
        codeWithMessage = "[\(result.statusCode)]\n\n" + message
    } else {
        codeWithMessage = message
    }
    log("error", message)
    let vc = KNAlertViewController(type: .error, message: codeWithMessage)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    vc.confirmClosure = completion
    
    topViewController()?.present(vc, animated: false, completion: nil)
}

@discardableResult
func alert(confirm: String,
           destructive: String,
           message: String,
           vcToPresent: UIViewController? = nil,
           _ completion: AlertViewClosure?,
           _ cancelCompletion: AlertViewClosure? = nil) -> KNAlertViewController {
    
    let vc = KNAlertViewController(type: .decision,
                                   message: message,
                                   confirm: confirm,
                                   destructive: destructive)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    vc.confirmClosure = completion
    vc.destructiveClosure = cancelCompletion
    
    // Todo to be removed when detecting the error of twice appearing in the pop-up.
    
    if let vcToPresent = vcToPresent {
        vcToPresent.present(vc, animated: false, completion: nil)
        
    } else {
        if let topV = topViewController() as? KNAlertViewController {
            return topV
        }
        topViewController()?.present(vc, animated: false, completion: nil)
    }
    
    
    
    return vc
}

@discardableResult
func alert(confirm: String, message: String, _ completion: AlertViewClosure?) -> KNAlertViewController {
    let vc = KNAlertViewController(type: .error,
                                   message: message,
                                   confirm: confirm, destructive: "")
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    vc.confirmClosure = completion
    
    // Todo to be removed when detecting the error of twice appearing in the pop-up.
    //@TODO what is this @bibi.
    if let topV = topViewController() as? KNAlertViewController {
        return topV
    }
    topViewController()?.present(vc, animated: false, completion: nil)
    
    return vc
}
@discardableResult

func alert(presentingBy controller: KNViewController,
           confirm: String,
           destructive: String,
           message: String,
           _ completion: AlertViewClosure?,
           _ cancelCompletion: AlertViewClosure? = nil) -> KNAlertViewController {
    
    let vc = KNAlertViewController(type: .decision,
                                   message: message,
                                   confirm: confirm,
                                   destructive: destructive)
    vc.modalPresentationStyle = .overFullScreen
    vc.modalTransitionStyle = .crossDissolve
    vc.confirmClosure = completion
    vc.destructiveClosure = cancelCompletion
    
    // Todo to be removed when detecting the error of twice appearing in the pop-up.
    if let topV = topViewController() as? KNAlertViewController {
        return topV
    }
    controller.present(vc, animated: false, completion: nil)
    
    return vc
}

