//
//  KNNavigationController.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

final class KNNavigationController: UINavigationController {
    
    // MARK: - Properties
    let textAttributes = [
        NSAttributedString.Key.foregroundColor: UIColor.black,
        NSAttributedString.Key.font: UIFont.medium(17)
    ]
    
    // MARK: - Lifecycle
    override func viewDidLoad() {
        super.viewDidLoad()
        
        setupUI()
    }
    
    // MARK: - UI
    private func setupUI() {
        navigationBar.setBackgroundImage(UIImage(), for: .default)
        navigationBar.shadowImage = UIImage()
        navigationBar.isTranslucent = false
        navigationBar.barTintColor = .white
        navigationBar.titleTextAttributes = textAttributes
        
        if #available(iOS 13.0, *) {
            let appearance = UINavigationBarAppearance()
            appearance.configureWithOpaqueBackground()
            appearance.backgroundColor = .white
            appearance.shadowColor = .clear
            appearance.titleTextAttributes = textAttributes

            navigationBar.standardAppearance = appearance
            navigationBar.scrollEdgeAppearance = appearance
        }
    }
    
    private func setupStatusBar() {
        let window = UIApplication.shared.windows.filter {$0.isKeyWindow}.first
        if let statusBarFrame = window?.windowScene?.statusBarManager?.statusBarFrame {
            let statusBarView = UIView(frame: statusBarFrame)
            statusBarView.backgroundColor = .clear
            
            view.addSubview(statusBarView)
        }
    }
}


