//
//  KNTextField.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
import UIKit

public class KNTextField: UITextField {
    
    // MARK: - Properties
    typealias KNTextFieldClosure = (String) -> ()
    
    var textChanged: KNTextFieldClosure  = { _ in }
    var textEndEditing: KNTextFieldClosure  = { _ in }
    
    var padding: UIEdgeInsets?
    
    // MARK: - Init
    required init(coder aDecoder: NSCoder) {
        super.init(coder: aDecoder)!
        
        setupUI()
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        setupUI()
    }
    
    func setupUI() {
        borderStyle = .none
        backgroundColor = .clear
        tintColor = .main
        
        padding = .zero
        
        autocorrectionType = .no
        textAlignment = .left
        
    }
    
    // MARK: - Handler
    func bind(callback: @escaping KNTextFieldClosure) {
        textChanged = callback
        
        addTarget(self, action: #selector(textFieldDidChange), for: .editingChanged)
    }
    
    @objc func textFieldDidChange(_ textField: KNTextField) {
        textChanged(textField.text ?? "")
    }
    
    func endEditingBind(callback: @escaping KNTextFieldClosure) {
        textEndEditing = callback
        
        addTarget(self, action: #selector(textFieldDidEndEditing), for: .editingDidEnd)
    }
    
    @objc func textFieldDidEndEditing(_ textField: KNTextField) {
        textEndEditing(textField.text ?? "")
    }
}

final class KNPaddingTextField: KNTextField {
    
    // MARK: - Init
    override func setupUI() {
        let padding_value: CGFloat = 0
        let padding_leftRight_value: CGFloat = 16
        padding = UIEdgeInsets(top: padding_value,
                               left: padding_leftRight_value,
                               bottom: padding_value,
                               right: padding_leftRight_value)
        super.setupUI()
    }
    
    override func textRect(forBounds bounds: CGRect) -> CGRect {
        let rect = super.textRect(forBounds: bounds)
        return rect.inset(by: padding ?? .zero)
    }
    
    override func editingRect(forBounds bounds: CGRect) -> CGRect {
        let rect = super.editingRect(forBounds: bounds)
        return rect.inset(by: padding ?? .zero)
    }
}

final class MATextFieldCode: KNTextField {
    // MARK: - Properties
    typealias DeleteFieldClosure = () -> ()

    var deleteButtonPressed: DeleteFieldClosure  = {
        //
    }
    
    // MARK: - Init
    override func setupUI() {
        autocorrectionType = .no
        textAlignment = .center
        tintColor = .main
    }
    
    override func deleteBackward() {
        deleteButtonPressed()
        super.deleteBackward()
    }
}
