//
//  KNViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 29/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

class KNViewController: UIViewController, Nameable {
    
    // MARK: - Properties
    
    var screenWidth: CGFloat {
        view.frame.width
    }
    
    var isPresented: Bool {
        if let navigationController = navigationController {
            return navigationController.viewControllers.firstIndex(of: self) == 0
        } else {
            return presentingViewController != nil
        }
    }
    
    var isNavigationBarHidden: Bool {
        navigationController?.navigationBar.isHidden ?? false
    }
    
    var isBackButtonHidden: Bool = false {
        didSet {
            navigationItem.hidesBackButton = isBackButtonHidden
        }
    }
    
    var hasCustomNavigation: Bool = false {
        didSet {
            navigationController?.setNavigationBarHidden(hasCustomNavigation, animated: true)
        }
    }

    
    
    var isTabbarHidden: Bool = false {
        didSet {
            tabBarController?.tabBar.isHidden = isTabbarHidden
        }
    }
    
    var statusBarStyle: UIStatusBarStyle = .default // default is black
    
    // MARK: - Init
    init() {
        let nibName = String(describing: type(of: self))
        super.init(nibName: nibName, bundle: nil)
    }
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    // MARK: - Lifecycle
    deinit {
        log("💥", Self.name)
    }
    
    override func viewDidLoad() {
        super.viewDidLoad()
        //
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        setNeedsStatusBarAppearanceUpdate()
    }
    
    override var preferredStatusBarStyle: UIStatusBarStyle {
        statusBarStyle
    }
}

// MARK: - Spinner
extension KNViewController {
    fileprivate func centerPoint() -> CGPoint {
        let x: CGFloat = (UIScreen.main.bounds.width / 2)
        var y: CGFloat = (UIScreen.main.bounds.height / 2)
        
        return CGPoint(x: x, y: y)
    }
    
    fileprivate func spinnerPosition() -> CGRect {
        let width: CGFloat = 100
        let height: CGFloat = 100
        
        let point = centerPoint()
        
        let frame = CGRect(x: point.x, y: point.y - (height / 2), width: width, height: height)
        
        return frame
    }
    
    @discardableResult
    func spin() -> KNSpinnerView {
        let spinner = KNSpinnerView(frame: spinnerPosition())
        spinner.backgroundColor = UIColor.clear
        spinner.alpha = 0
        
        view.addSubview(spinner)
        view.bringSubviewToFront(spinner)
        spinner.center = CGPoint(x: centerPoint().x, y: centerPoint().y)
        
        UIView.animate(withDuration: 0.3) {
            spinner.alpha = 1
        }
        
        return spinner
    }
    
    func customSpin() -> KNSpinnerView {
        let spinner = KNSpinnerView(frame: spinnerPosition())
        spinner.backgroundColor = .clear
        spinner.alpha = 0
        
        return spinner
    }
    
    func addSpinner(spinner: KNSpinnerView) {
        view.addSubview(spinner)
        view.bringSubviewToFront(spinner)
        spinner.center = CGPoint(x: centerPoint().x, y: centerPoint().y)
        
        UIView.animate(withDuration: 0.3) {
            spinner.alpha = 1
        }
    }
    
    func addSpinnerToView(spinner: KNSpinnerView, displayView: UIView) {
        displayView.addSubview(spinner)
        displayView.bringSubviewToFront(spinner)
        spinner.center = CGPoint(x: centerPoint().x, y: centerPoint().y)
        
        UIView.animate(withDuration: 0.3) {
            spinner.alpha = 1
        }
    }
}

