//
//  LoginViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

final class LoginViewController: KNViewController {
    // MARK: - Outlets
    @IBOutlet private weak var tableView: UITableView!
    
    // MARK: - Properties
    private weak var delegate: LoginViewControllerDelegate?
    var coordinator: AuthenticationCoordinator?
    
    private var modelDataSource: LoginDataSource?
    private var modelController: LoginModelController?
    var window: UIWindow?
    
    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        setupUI()
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        hasCustomNavigation = false
        
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        if let token = SecurityManager.shared.readToken() {
            let vc = LeaderViewController()
            self.navigationController?.pushViewController(vc, animated: false)
        }
    }
    
}
// MARK: - UI
extension LoginViewController {
    private func setupUI() {
        isBackButtonHidden = true
        setupTableView()
    }
    
    private func setupTableView() {
        modelDataSource = LoginDataSource()
        modelController = LoginModelController(delegate: self, viewController: self)
        
        modelDataSource?.outputProtocol = modelController
        modelController?.inputProtocol = modelDataSource
        
        modelController?.coordinator = self.coordinator
        
        tableView.delegate = modelDataSource
        tableView.dataSource = modelDataSource
        
        tableView.keyboardDismissMode = .interactive
        tableView.set(cells: [FieldTableViewCell.self, ButtonTableViewCell.self])
        tableView.reloadData()
    }
    
}

// MARK: - LoginViewControllerDelegate
extension LoginViewController: LoginViewControllerDelegate {
    func updateUI() {
        tableView.beginUpdates()
        tableView.endUpdates()
    }
}
