//
//  AuthenticationCoordinator.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
import UIKit

final class AuthenticationCoordinator: Coordinatable {
    // MARK: - Properties
    var navigationController: UINavigationController
    var step: AuthenticationStep?
    
    // MARK: - Init
    init(navigationController: UINavigationController) {
        self.navigationController = navigationController
    }
    
    // MARK: - Next/Previous Step
    func next() {
        guard let step = step else { return }
        
        switch step {
        case .signin:
            signin()
        }
    }
    
    // MARK: - Start
    func start() {
        guard let step = step else { return }
        
        if step != .signin {
            self.signin()
        } else {
            self.continue()
        }
    }
    
    func finish() {
        navigationController.popToRootViewController(animated: true)
    }
    
    func previous() {
        navigationController.popViewController(animated: true)
    }
    
    func cancel() {
        navigationController.popToRootViewController(animated: true)
    }
    
    func `continue`() {
        guard let step = step else { return }
        
        switch step {
        case .signin:
            let signin = LoginViewController()
            signin.coordinator = self
            
            navigationController.push(viewControllers: [signin], animated: true)
            
        }
    }
    
    // MARK: - signin
    private func signin() {
        let signinViewController = LoginViewController()
        signinViewController.coordinator = self
        navigationController.pushViewController(signinViewController, animated: true)
    }
}

