//
//  LoginDataSource.swift
//  TraccarClient
//
//  Created by George Makhoul on 26/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
import Foundation

final class LoginDataSource: NSObject {
    
    // MARK: - Properties
    private var fields: [KNField] = []
    weak var outputProtocol: LoginOutputProtocol?
    
    // MARK: - Init
    override init() {
        //
    }
}

// MARK: - TableView
extension LoginDataSource: UITableViewDelegate, UITableViewDataSource {
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        fields.count
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let field = fields[indexPath.row]
        
        if field.type == .text {
            let cell = tableView.dequeue(withCell: FieldTableViewCell.self)
            cell.delegate = self
            
            cell.set(field: field, indexPath: indexPath)
            return cell
        }
        
        if field.type == .button {
            let cell = tableView.dequeue(withCell: ButtonTableViewCell.self)
            cell.delegate = self
            cell.set(field: field, indexPath: indexPath)
            return cell
        }
        
        return UITableViewCell()
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        return fields[indexPath.row].height
    }
}


// MARK: - Input Protocol
extension LoginDataSource: LoginInputProtocol {
    func setFields(fields: [KNField]) {
        self.fields = fields
        
        outputProtocol?.reloadUI()
    }
}

// MARK: - Field Protocok
extension LoginDataSource: KNTextFieldViewDelegate {
    func updateUI(for field: KNField?) {
        outputProtocol?.reloadUI()
    }
}

// MARK: - Button Delegate
extension LoginDataSource: ButtonTableViewCellDelegate {
    func cellButtonTapped(action: KNButtonViewAction) {
        outputProtocol?.cellButtonTapped(action: action)
    }
}
