//
//  ShiftsView.swift
//  TraccarClient
//
//  Created by George Makhoul on 19/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
protocol ShiftViewDelegate {
    func startTime()
}

final class ShiftsView: KNComponentView {
    // MARK: - Outlets
    @IBOutlet var view: UIView!
    @IBOutlet private weak var connectionStatusLabel: UILabel!
    @IBOutlet private weak var descLabel1: UILabel!
    @IBOutlet private weak var descLabel2: UILabel!
    @IBOutlet private weak var timeRemaining: UILabel!
    
    @IBOutlet private weak var workingDaysLabel: UILabel!
    @IBOutlet private weak var daysStack: UIStackView!
    @IBOutlet private weak var startLabel: UILabel!
    @IBOutlet private weak var endLabel: UILabel!
    @IBOutlet private weak var startTimeLabel: UILabel!
    @IBOutlet private weak var endTimeLabel: UILabel!
    @IBOutlet private weak var startTimeView: UIView!
    @IBOutlet private weak var endTimeView: UIView!
    @IBOutlet private weak var sundayLabel: UILabel!
    @IBOutlet private weak var mondayLabel: UILabel!
    @IBOutlet private weak var tuesdayLabel: UILabel!
    @IBOutlet private weak var wednesdayLabel: UILabel!
    @IBOutlet private weak var thursdayLabel: UILabel!
    @IBOutlet private weak var fridayLabel: UILabel!
    @IBOutlet private weak var saturdayLabel: UILabel!
    @IBOutlet private weak var startDateLabel_width: NSLayoutConstraint!
    @IBOutlet private weak var endDateLabel_width: NSLayoutConstraint!
    
    // MARK: - Properties
    private var shiftModel = ShiftModel()
    private var days: [UILabel] = []
    var timer: Timer?
    var remainingTime: TimeInterval = 0
    var remainingToEnd: TimeInterval = 0
    var delegate: ShiftViewDelegate?
    
    // MARK: - Lifecycle
    override func setupNib() {
        super.setupNib()
        
        view.frame = bounds
        view.autoresizingMask = [.flexibleHeight, .flexibleWidth]
        
        setupUI()
    }
    
    private func setupUI() {
        
        days = [sundayLabel, mondayLabel, tuesdayLabel, wednesdayLabel, thursdayLabel, fridayLabel, saturdayLabel]
        
        initDays(days: days)
        
        connectionStatusLabel.set(text: "Tracking stopped.", color: .black, font: .bold(20))
        connectionStatusLabel.textAlignment = .center
        
        descLabel1.set(text: "Tracking will start after", color: .black, font: .regular(16))
        descLabel1.textAlignment = .center
        
        timeRemaining.set(text: "2 hours 46 minutes", color: .black, font: .bold(16))
        timeRemaining.textAlignment = .center
        
        descLabel2.set(text: "at the beginning of your shift", color: .black, font: .regular(16))
        descLabel2.textAlignment = .center
        
        workingDaysLabel.set(text: "Working Days", color: .black, font: .medium(20))
        workingDaysLabel.textAlignment = .center
        
        daysStack.distribution = .fillEqually
        daysStack.spacing = 10
        
        startLabel.set(text: "Shift Starts at", color: .black, font: .medium(18))
        endLabel.set(text: "Shift Ends at", color: .black, font: .medium(18))
        
        startTimeView.addCorners(4)
        endTimeView.addCorners(4)
        
        startTimeView.backgroundColor = .invalid
        endTimeView.backgroundColor = .invalid
        
        startTimeLabel.set(text: "", color: .invalidText, font: .semibold(13))
        endTimeLabel.set(text: "", color: .invalidText, font: .semibold(13))
    }
    
    @objc func updateTimeLabel() {
        if remainingTime > 0 {
            let hours = Int(remainingTime) / 3600
            let minutes = (Int(remainingTime) % 3600) / 60
            timeRemaining.text = "\(hours) hours \(minutes) minutes"
            remainingTime -= 60 // Subtract 60 seconds (1 minute)
            
        } else {
            timer?.invalidate()
            delegate?.startTime()
        }
    }
    
    private func initDays(days: [UILabel]) {
        for day in days {
            day.layer.cornerRadius = 10
            day.clipsToBounds = true
            day.set(color: .invalidText)
            day.backgroundColor = .invalid
        }
    }
    
    private func setShiftDays(timings: [TimingModel]) {
        for day in days {
            for timing in timings {
                for id in timing.daysOfWeek {
                    if day.restorationIdentifier == id {
                        day.set(color: .white)
                        day.backgroundColor = .secondary
                    }
                    if day.restorationIdentifier?.lowercased() == getDayName().lowercased() {
                        day.set(color: .secondary)
                        day.backgroundColor = .systemGreen
                        day.addBorder(radius: 10, width: 2, color: .secondary)
                    }
                }
            }
        }
    }
    
    @IBAction func dayTapped(_ sender: UIButton) {
        deselectDays()
        for day in days {
            if day.restorationIdentifier?.lowercased() == sender.restorationIdentifier?.lowercased() {
                day.addBorder(radius: 10, width: 2, color: getDayName().lowercased() == day.restorationIdentifier?.lowercased() ? .secondary : .systemGreen)
                if shiftModel.getStartTimeForToday(dayName: day.restorationIdentifier?.lowercased() ?? "") == "Weekend" {
                    timeRemaining.text = "It's your weekend, Enjoy!"
                    startTimeLabel.text = "Weekend"
                    startDateLabel_width.constant = 200
                    
                    endTimeView.hide()
                    startLabel.text = ""
                    endLabel.text = ""
                    descLabel1.text = ""
                    descLabel2.text = ""
                }
                else {
                    timeRemaining.text = "It's your weekend, Enjoy!"
                    startDateLabel_width.constant = 55
                    startLabel.text = "Shift Starts at"
                    endLabel.text = "Shift Ends at"
                    endTimeView.show()
                    startTimeLabel.set(text: shiftModel.getStartTimeForToday(dayName: day.restorationIdentifier?.lowercased() ?? ""))
                    startTimeLabel.textAlignment = .center
                    
                    endTimeLabel.set(text: shiftModel.getEndTimeForToday(dayName: day.restorationIdentifier?.lowercased() ?? ""))
                    endTimeLabel.textAlignment = .center
                }
            }
        }
        
    }
    
    private func deselectDays() {
        for day in days {
            day.addBorder(radius: 10, width: 0, color: .clear)
        }
    }
 
    private func startTimer(model: ShiftModel) {
        let dateFormatter = DateFormatter()
        dateFormatter.dateFormat = "HH:mm"
        
        var currentTime = Date()
        let time = dateFormatter.string(from: Date())
        if let current = dateFormatter.date(from: time) {
            currentTime = current
        }
        
        if model.getStartTimeForToday() == "Weekend" {
            timeRemaining.text = "It's your weekend, Enjoy!"
            startTimeLabel.text = "Weekend"
            startDateLabel_width.constant = 200
            
            endTimeView.hide()
            startLabel.text = ""
            endLabel.text = ""
            descLabel1.text = ""
            descLabel2.text = ""
            return
        }
        
        var targetTime = dateFormatter.date(from: model.getStartTimeForToday())!
        // if targetTime is earlier than currentTime, add 1 day to targetTime
        if targetTime < currentTime {
            targetTime = Calendar.current.date(byAdding: .day, value: 1, to: targetTime)!
        }
        
        remainingTime = targetTime.timeIntervalSince(currentTime)
        
        timer = Timer.scheduledTimer(timeInterval: 60, target: self, selector: #selector(updateTimeLabel), userInfo: nil, repeats: true)
        RunLoop.current.add(timer!, forMode: RunLoop.Mode.common)
        timer?.fire()
    }
    
    func set(model: ShiftModel, delegate: ShiftViewDelegate) {
        self.delegate = delegate
        self.shiftModel = model
        setShiftDays(timings: model.timings)
        
        startTimeLabel.set(text: model.getStartTimeForToday())
        startTimeLabel.textAlignment = .center
        
        endTimeLabel.set(text: model.getEndTimeForToday())
        endTimeLabel.textAlignment = .center
        
        startTimer(model: model)
    }
}
