//
//  Network+Endpoints.swift
//  traccar
//
//  Created by George Makhoul on 10/25/23.
//

import Foundation

// MARK: - API Endpoint
enum Endpoint: String, CaseIterable {
    
    static var baseURL: String {
        switch app.environment {
        case .development, .sandbox:
            return "\(serverURL)/api/"
        case .production:
            return "\(serverURL)/api/"
        }
    }
    
    static var serverURL: String {
        switch app.environment {
        case .development, .sandbox:
            return "https://dev-nc-teams-api.nmo.ai"
        case .production:
            return "https://me-nc-teams-api.nmo.ai"
        }
    }
    
    case all
    case signin = "fleet-login"
    case logout = "logout"
    case permissions = "nextcloud/fleet/permissions"
    case parameters = "nextcloud/fleet/tracking-parameters"
    case check = "user/check-token"
    case shift = "shift/my-shift"

    // ///////////////
    
    // get endpoint with resource
    func with(resource: Int) -> String {
        String(format: self.rawValue, "\(resource)")
    }
	
	func with(detailsID: Int, productID: Int) -> String {
		String(format: self.rawValue, "\(detailsID)", "\(productID)")
	}
    
    func with(productBuilderID: Int, variantID: Int) -> String {
        String(format: self.rawValue, "\(productBuilderID)", "\(variantID)")
    }
    
    func with(orderID: Int, lineID: Int) -> String {
        String(format: self.rawValue, "\(orderID)", "\(lineID)")
    }
    
    func with(basket: Int, line: Int) -> String {
        String(format: self.rawValue, "\(basket)", "\(line)")
    }
    
    func with(resource: String) -> String {
        String(format: self.rawValue, "\(resource)")
    }
    
    func with(product: Int, comment: Int) -> String {
        String(format: self.rawValue, "\(product)", "\(comment)")
    }
    
    func with(resource: String, urlParams: String) -> String {
        var result: String = ""
        
        if urlParams.isEmpty {
            result = with(resource: resource)
        } else {
            if urlParams.contains(resource) {
                result = result + "?\(urlParams)"
            } else {
                result = with(resource: resource)
                result = result + "&\(urlParams)"
            }
        }
        
        return result
        
    }
    func with(resource: Int, urlParams: String) -> String {
        var result: String = ""
        
        result = with(resource: resource)
        
        if !urlParams.isEmpty {
            result = result + "?\(urlParams)"
        }
        
        return result
        
    }
    
    func with(urlParams: String) -> String {
        var result: String = ""
        
        result = String(format: self.rawValue, "")
        
        if !urlParams.isEmpty {
            
            var params = "?"
            
            if let ampersand = urlParams.first {
                if ampersand == "&" {
                    params += String(urlParams.dropFirst())
                } else {
                    params += urlParams
                }
            }
            
            result.append(params)
            
        }
        
        return result
    }
    
}

