//
//  PermissionModel.swift
//  TraccarClient
//
//  Created by George Makhoul on 07/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
final class PermissionsModel: KNObject {
    
    // MARK: - Properties
    var update_settings: Bool = false
    var tracking_auto: Bool = false
    
    // MARK: - Init
    override init() {
        super.init()
    }
    
    init(_ dict: JSON, save: Bool = false) {
        super.init()
        update_settings = dict["user_can_update_his_settings"] as? Bool ?? false
        tracking_auto = dict["turn_on_tracking_automatically"] as? Bool ?? false
        
        if save {
            saveToDefaults()
        }
    }
    //MARK: - Methods
    private func saveToDefaults() {
        let ud = UserDefaults.standard
        ud.set(update_settings, forKey: Keys.User.updateSettings)
        ud.set(tracking_auto, forKey: Keys.User.autoTracking)
    }
    
    class func loadFromDefaults() -> PermissionsModel {
        let ud = UserDefaults.standard
        var params: PARAMS = [:]
        
        if let updateSettings = ud.value(forKey: Keys.User.updateSettings) as? Bool {
            params.add(key: .updateSettings, updateSettings)
        }
        
        if let autoTracking = ud.value(forKey: Keys.User.autoTracking) as? Bool {
            params.add(key: .autoTracking, autoTracking)
        }
        
        return PermissionsModel(params)
    }
    
    class func clearUserFromDefaults() {
        let ud = UserDefaults.standard
        ud.removeObject(forKey: Keys.User.updateSettings)
        ud.removeObject(forKey: Keys.User.autoTracking)
    }
}
