//
//  SettingsSegmentTableViewCell.swift
//  TraccarClient
//
//  Created by George Makhoul on 01/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
protocol SettingsSegmentTableViewCellDelegate {
    func didUpdateSegment(model: DefaultsModel)
}

class SettingsSegmentTableViewCell: KNTableViewCell {
    // MARK: - Outlets
    @IBOutlet weak var segments: UISegmentedControl!
    @IBOutlet weak var titleLabel: UILabel!
    
    // MARK: - Properties
    let defaults = UserDefaults.standard
    var index: Int = 0
    var delegate: SettingsSegmentTableViewCellDelegate?
    var model: DefaultsModel?
    
    // MARK: - LifeCycle
    override func awakeFromNib() {
        super.awakeFromNib()
        setupUI()
    }
    
    private func setupUI() {
        titleLabel.set(text: "", color: .main, font: .semibold(16))
        let selectedAttributes: [NSAttributedString.Key: Any] = [
            .foregroundColor: UIColor.white
        ]
        segments.setTitleTextAttributes(selectedAttributes, for: .selected)
        segments.addTarget(self, action: #selector(segmentChanged), for: .valueChanged)
    }
    
    @objc private func segmentChanged(_ sender: UISegmentedControl) {
        switch sender.selectedSegmentIndex {
        case 0:
            model?.value = "low"
            
        case 1:
            model?.value = "medium"
            
        case 2:
            model?.value = "high"

        default:
            model?.value = "medium"
        }
        delegate?.didUpdateSegment(model: model ?? DefaultsModel())
    }
    
    func set(defaultValue: DefaultsModel, index: Int) {
        self.index = 0
        self.model = defaultValue
        
        titleLabel.set(text: defaultValue.title)
        let selected = defaultValue.value as? String ?? ""
        if selected == "high" {
            segments.selectedSegmentIndex = 2
            
        } else if selected == "medium" {
            segments.selectedSegmentIndex = 1
            
        } else {
            segments.selectedSegmentIndex = 0
        }
    }
}
