//
//  SettingsSwitchTableViewCell.swift
//  TraccarClient
//
//  Created by George Makhoul on 01/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

protocol SettingsSwitchTableViewCellDelegate {
    func didUpdateSwitch(model: DefaultsModel)
}

class SettingsSwitchTableViewCell: KNTableViewCell {
    // MARK: - Outlets
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var switchView: UISwitch!
    
    // MARK: - Properties
    let defaults = UserDefaults.standard
    var index: Int = 0
    var model: DefaultsModel?
    var delegate: SettingsSwitchTableViewCellDelegate?
    
    // MARK: - LifeCycle
    override func awakeFromNib() {
        super.awakeFromNib()
        setupUI()
    }
    
    private func setupUI() {
        titleLabel.set(text: "", color: .main, font: .semibold(16))
        switchView.addTarget(self, action: #selector(switchChanged), for: .valueChanged)
    }
    
    @objc private func switchChanged(_ sender: UISwitch) {
        model?.value = sender.isOn
        delegate?.didUpdateSwitch(model: model ?? DefaultsModel())
    }
    
    func set(defaultValue: DefaultsModel, index: Int) {
        self.index = index
        self.model = defaultValue
        
        titleLabel.set(text: defaultValue.title)
        switchView.isOn = defaultValue.value as? Bool ?? false
    }
}
