//
//  SettingsTextTableViewCell.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit
protocol SettingsTextTableViewCellDelegate {
    func didUpdateText(model: DefaultsModel)
}

class SettingsTextTableViewCell: KNTableViewCell {
    // MARK: - Outlets
    @IBOutlet private weak var titleLabel: UILabel!
    @IBOutlet private weak var valueField: UITextField!
    
    // MARK: - Properties
    let defaults = UserDefaults.standard
    var index: Int = 0
    var model: DefaultsModel?
    var delegate: SettingsTextTableViewCellDelegate?
    
    // MARK: - LifeCycle
    override func awakeFromNib() {
        super.awakeFromNib()
        setupUI()
    }
    
    private func setupUI() {
        titleLabel.set(text: "", color: .main, font: .semibold(16))
        
        valueField.textAlignment = .center
        valueField.textColor = .black
        valueField.font = .semibold(16)
        valueField.addTarget(self, action: #selector(textChanged), for: .editingChanged)
    }
    
    func set(defaultValue: DefaultsModel, index: Int) {
        self.index = index
        self.model = defaultValue
        
        titleLabel.set(text: defaultValue.title)
        valueField.text = defaultValue.value as? String ?? ""
        valueField.keyboardType = defaultValue.keyboardType
    }
    @objc func textChanged(_ sender: UITextField) {
        model?.value = sender.text
        delegate?.didUpdateText(model: model ?? DefaultsModel())
    }
}
