//
//  ShiftsModel.swift
//  TraccarClient
//
//  Created by George Makhoul on 14/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation

final class ShiftModel: KNObject {
    // MARK: - Properties
    var id: Int = 0
    var name: String = ""
    var minHours: Int?
    var maxHours: Int?
    var cycle: Int?
    var isHourly: Bool = false
    var workDays: [String] = []
    var shiftTypeId: Int = 0
    var isDefault: Bool = false
    var timings: [TimingModel] = []
    var assignees: [String] = []

    // MARK: - Init
    override init() {
        super.init()
    }

    init(_ dict: JSON) {
        super.init()

        id = dict["id"] as? Int ?? 0
        name = dict["name"] as? String ?? ""
        minHours = dict["min_hours"] as? Int
        maxHours = dict["max_hours"] as? Int
        cycle = dict["cycle"] as? Int
        isHourly = dict["is_hourly"] as? Bool ?? false
        workDays = dict["work_days"] as? [String] ?? []
        shiftTypeId = dict["shift_type_id"] as? Int ?? 0
        isDefault = dict["is_default"] as? Bool ?? false
        assignees = dict["assignees"] as? [String] ?? []

        if let timingsArray = dict["timings"] as? [JSON] {
            timings = timingsArray.map { TimingModel($0) }
        }
    }
    
    func getStartTimeForToday(dayName: String = "") -> String {
        for item in timings {
            for day in item.daysOfWeek {
                if dayName == "" {
                    if getDayName().lowercased() == day.lowercased() {
                        return item.startsAt
                    }
                } else {
                    if dayName.lowercased() == day.lowercased() {
                        return item.startsAt
                    }
                }
            }
        }
        return "Weekend"
    }
    
    func getEndTimeForToday(dayName: String = "") -> String {
        for item in timings {
            for day in item.daysOfWeek {
                if dayName == "" {
                    if getDayName().lowercased() == day.lowercased() {
                        return item.endsAt
                    }
                } else {
                    if dayName.lowercased() == day.lowercased() {
                        return item.endsAt
                    }
                }
            }
        }
        return "Weekend"
    }
}

final class TimingModel: KNObject {
    // MARK: - Properties
    var id: Int = 0
    var daysOfWeek: [String] = []
    var startsAt: String = ""
    var endsAt: String = ""
    var arrivalGracePeriod: Int = 0
    var leavingGracePeriod: Int = 0
    var breaks: String?
    var shiftId: Int = 0
    var createdAt: String = ""
    var updatedAt: String = ""

    // MARK: - Init
    override init() {
        super.init()
    }

    init(_ dict: JSON) {
        super.init()

        id = dict["id"] as? Int ?? 0
        
        var startStr = dict["starts_at"] as? String ?? ""
        startStr.insert(":", at: startStr.index(startStr.startIndex, offsetBy: 2))
        startsAt = startStr
        
        var endStr = dict["ends_at"] as? String ?? ""
        endStr.insert(":", at: endStr.index(endStr.startIndex, offsetBy: 2))
        endsAt = endStr
        
        arrivalGracePeriod = dict["arrival_grace_period"] as? Int ?? 0
        leavingGracePeriod = dict["leaving_grace_period"] as? Int ?? 0
        breaks = dict["breaks"] as? String
        shiftId = dict["shift_id"] as? Int ?? 0
        createdAt = dict["created_at"] as? String ?? ""
        updatedAt = dict["updated_at"] as? String ?? ""

        if let daysOfWeekArray = dict["days_of_week"] as? [String] {
            daysOfWeek = daysOfWeekArray
        }
    }
}

