//
//  SignInModel.swift
//  TraccarClient
//
//  Created by George Makhoul on 29/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
final class SignInModel: KNObject {
    
    // MARK: - Properties
    var token: String = ""
    var user: UserModel = UserModel([:])
    var device: DeviceModel = DeviceModel([:])
    
    // MARK: - Init
    override init() {
        super.init()
    }
    
    init(_ dict: JSON, saveUser: Bool = false) {
        super.init()
        token = dict["token"] as? String ?? ""
        
        if let user = dict["user"] as? JSON {
            self.user = UserModel(user, saveUser: true)
        }
        
        if let device = dict["device"] as? JSON {
            self.device = DeviceModel(device, saveUser: true)
        }
        
        if saveUser {
            saveToUserDefaults()
        }
    }
    //MARK: - Methods
    private func saveToUserDefaults() {
        let ud = UserDefaults.standard
        ud.set(token, forKey: Keys.User.access_token)
    }
    
    class func loadFromUserDefaults() -> SignInModel {
        let ud = UserDefaults.standard
        var params: PARAMS = [:]
        
        if let token = ud.value(forKey: Keys.User.access_token) as? Int {
            params.add(key: .access_token, token)
        }
        
        return SignInModel(params)
    }
    
    class func clearUserFromUserDefaults() {
        let ud = UserDefaults.standard
        ud.removeObject(forKey: Keys.User.access_token)
    }
}
