//
//  TrackingParametersModel.swift
//  TraccarClient
//
//  Created by George Makhoul on 07/11/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
final class TrackingParametersModel: KNObject {
    
    // MARK: - Properties
    var frequency: Int = 0
    var distance: Int = 0
    var angle: Int = 0
    var offline_buffering: Bool = false
    var accuracy: String = ""
    
    // MARK: - Init
    override init() {
        super.init()
    }
    
    init(_ dict: JSON, save: Bool = false) {
        super.init()
        frequency = dict["frequency_in_seconds"] as? Int ?? 0
        distance = dict["distance_in_meters"] as? Int ?? 0
        angle = dict["angle_in_degrees"] as? Int ?? 0
        offline_buffering = dict["offline_buffering"] as? Bool ?? false
        accuracy = dict["accuracy"] as? String ?? ""
        
        if save && !UserDefaults.standard.bool(forKey: "params_changed"){
            saveToDefaults()
        }
    }
    //MARK: - Methods
    private func saveToDefaults() {
        let ud = UserDefaults.standard
        ud.set(frequency, forKey: "frequency_preference")
        ud.set(distance, forKey: "distance_preference")
        ud.set(angle, forKey: "angle_preference")
        ud.set(offline_buffering, forKey: "buffer_preference")
        ud.set(accuracy, forKey: "accuracy_preference")
    }
    
    class func loadFromDefaults() -> TrackingParametersModel {
        let ud = UserDefaults.standard
        var params: PARAMS = [:]
        
        if let frequecy = ud.value(forKey: "frequency_preference") as? Int {
            params.add(key: .frequency, frequecy)
        }
        
        if let distance = ud.value(forKey: "distance_preference") as? Int {
            params.add(key: .distance, distance)
        }
        if let angle = ud.value(forKey: "angle_preference") as? Int {
            params.add(key: .angle, angle)
        }
        if let offlineBuffering = ud.value(forKey: "buffer_preference") as? Bool {
            params.add(key: .offline_buffering, offlineBuffering)
        }
        if let accuracy = ud.value(forKey: "accuracy_preference") as? String {
            params.add(key: .accuracy, accuracy)
        }
        
        return TrackingParametersModel(params)
    }
    
    class func clearUserFromDefaults() {
        let ud = UserDefaults.standard
        ud.removeObject(forKey: "frequency_preference")
        ud.removeObject(forKey: "distance_preference")
        ud.removeObject(forKey: "angle_preference")
        ud.removeObject(forKey: "buffer_preference")
        ud.removeObject(forKey: "accuracy_preference")
    }
}

