//
//  TransactionsViewController.swift
//  TraccarClient
//
//  Created by George Makhoul on 31/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

class TransactionsViewController: KNViewController {
    // MARK: - Outlets
    @IBOutlet weak var customHeaderView: CustomHeaderView!
    @IBOutlet weak var noDataFoundLabel: UILabel!
    @IBOutlet weak var tableView: UITableView!
    
    // MARK: - Properties
    static let LIMIT = 20
    static var transactionViewController: TransactionsViewController?
    
    static var messages = [String]()

    // MARK: - LifeCycle
    override func viewDidLoad() {
        super.viewDidLoad()
        noDataFoundLabel.set(text: "No Transactions to display yet", color: .main, font: .semibold(16))
        
        customHeaderView.set(type: .status, delegate: self)
        setupTableView()
    }
    
    
   private func setupTableView() {
       tableView.delegate = self
       tableView.dataSource = self
       tableView.register(cell: TransactionTableViewCell.self)
       tableView.separatorColor = .clear
       tableView.allowsSelection = false
       tableView.separatorStyle = .none
       tableView.reloadData()
    }
    
    // MARK: - Methods
    class func addMessage(_ message: String) {
        DispatchQueue.main.async {
            let formatter = DateFormatter()
            formatter.dateFormat = "HH:mm:ss - "
            
            self.messages.append(formatter.string(from: Date()) + (message))
            
            if self.messages.count > self.LIMIT {
                self.messages.remove(at: 0)
            }
            self.transactionViewController?.tableView.reloadData()
            self.transactionViewController?.noDataFoundLabel.isHidden = !TransactionsViewController.messages.isEmpty
        }
    }
    
    
    
    class func clearMessages() {
        DispatchQueue.main.async {
            self.messages.removeAll()
            self.transactionViewController?.tableView.reloadData()
            self.transactionViewController?.noDataFoundLabel.isHidden = !TransactionsViewController.messages.isEmpty
        }
    }
    
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        noDataFoundLabel.isHidden = !TransactionsViewController.messages.isEmpty
    }
    
    override func viewDidAppear(_ animated: Bool) {
        super.viewDidAppear(animated)
        TransactionsViewController.transactionViewController = self
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        TransactionsViewController.transactionViewController = nil
    }
}

// MARK: - UITableViewDelegate
extension TransactionsViewController: UITableViewDelegate, UITableViewDataSource {
    
    func numberOfSections(in tableView: UITableView) -> Int {
        1
    }
    
    func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        TransactionsViewController.messages.count
    }
    
    func tableView(_ tableView: UITableView, heightForRowAt indexPath: IndexPath) -> CGFloat {
        40
    }
    
    func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeue(withCell: TransactionTableViewCell.self)
        cell.set(status: TransactionsViewController.messages[indexPath.row])
        return cell
    }
}

// MARK: -
extension TransactionsViewController: CustomHeaderViewDelegate {
    func rightButtonTapped(action: RightButtonAction) {
        TransactionsViewController.clearMessages()
        noDataFoundLabel.isHidden = !TransactionsViewController.messages.isEmpty
    }
    
    func leftButtonTapped(action: LeftButtonAction) {
        self.navigationController?.popViewController(animated: true)
    }
}
