//
//  UIFont.swift
//  TraccarClient
//
//  Created by George Makhoul on 25/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import Foundation
import UIKit

extension UIFont {
    
    // MARK: - Font Name
    enum AppFont {
        case black
        case blackItalic
        case bold
        case boldItalic
        case extraBold
        case extraBoldItalic
        case extraLight
        case extraLightItalic
        case italic
        case light
        case lightItalic
        case medium
        case mediumItalic
        case regular
        case semibold
        case semiboldItalic
        case thin
        case thinitalic

        enum fonts: String {
            case black = "Montserrat-Black"
            case blackItalic = "Montserrat-BlackItalic"
            case bold = "Montserrat-Bold"
            case boldItalic = "Montserrat-BoldItalic"
            case extraBold = "Montserrat-ExtraBold"
            case extraBoldItalic = "Montserrat-ExtraBoldItalic"
            case extraLight = "Montserrat-ExtraLight"
            case extraLightItalic = "Montserrat-ExtraLightItalic"
            case italic = "Montserrat-Italic"
            case light = "Montserrat-Light"
            case lightItalic = "Montserrat-LightItalic"
            case medium = "Montserrat-Medium"
            case mediumItalic = "Montserrat-MediumItalic"
            case regular = "Montserrat-Regular"
            case semibold = "Montserrat-Semibold"
            case semiboldItalic = "Montserrat-SemiboldItalic"
            case thin = "Montserrat-Thin"
            case thinitalic = "Montserrat-ThinItalic"
        }

        func font() -> String {
            switch self {
            case .bold:
                return fonts.bold.rawValue
            case .semibold:
                return fonts.semibold.rawValue
            case .regular:
                return fonts.regular.rawValue
            case .medium:
                return fonts.medium.rawValue
            case .light:
                return fonts.light.rawValue
            case .black:
                return fonts.black.rawValue
            case .blackItalic:
                return fonts.blackItalic.rawValue
            case .boldItalic:
                return fonts.boldItalic.rawValue
            case .extraBold:
                return fonts.extraBold.rawValue
            case .extraBoldItalic:
                return fonts.extraBoldItalic.rawValue
            case .extraLight:
                return fonts.extraLight.rawValue
            case .extraLightItalic:
                return fonts.extraLightItalic.rawValue
            case .italic:
                return fonts.italic.rawValue
            case .lightItalic:
                return fonts.lightItalic.rawValue
            case .mediumItalic:
                return fonts.mediumItalic.rawValue
            case .semiboldItalic:
                return fonts.semiboldItalic.rawValue
            case .thin:
                return fonts.thin.rawValue
            case .thinitalic:
                return fonts.thinitalic.rawValue
            }
        }
    }
}

extension UIFont {
    
    // MARK: - Get Font
    static func getFont(_ name: UIFont.AppFont, size: CGFloat) -> UIFont {
        return getFont(name: name.font(), size: size)
    }
    
    static func getFont(name: String, size: CGFloat) -> UIFont {
        if let font = UIFont(name: name, size: size) {
            return font
        } else {
            return UIFont.systemFont(ofSize: size)
        }
    }
    
    // MARK: - Font Styles
    class func light(_ size: CGFloat) -> UIFont {
        getFont(.light, size: size)
    }
    
    class func regular(_ size: CGFloat) -> UIFont {
        getFont(.regular, size: size)
    }
    
    class func medium(_ size: CGFloat) -> UIFont {
        getFont(.medium, size: size)
    }
    
    class func semibold(_ size: CGFloat) -> UIFont {
        getFont(.semibold, size: size)
    }
    
    class func bold(_ size: CGFloat) -> UIFont {
        getFont(.bold, size: size)
    }
    
    class func black(_ size: CGFloat) -> UIFont {
        getFont(.black, size: size)
    }
    
    class func blackItalic(_ size: CGFloat) -> UIFont {
        getFont(.blackItalic, size: size)
    }
    
    class func extraLightItalic(_ size: CGFloat) -> UIFont {
        getFont(.extraLightItalic, size: size)
    }
    
    class func italic(_ size: CGFloat) -> UIFont {
        getFont(.italic, size: size)
    }
    
    class func lightItalic(_ size: CGFloat) -> UIFont {
        getFont(.lightItalic, size: size)
    }
    
    class func mediumItalic(_ size: CGFloat) -> UIFont {
        getFont(.mediumItalic, size: size)
    }
    
    class func semiboldItalic(_ size: CGFloat) -> UIFont {
        getFont(.semiboldItalic, size: size)
    }
    
    class func thin(_ size: CGFloat) -> UIFont {
        getFont(.thin, size: size)
    }
    
    class func thinitalic(_ size: CGFloat) -> UIFont {
        getFont(.thinitalic, size: size)
    }
    
    class func boldItalic(_ size: CGFloat) -> UIFont {
        getFont(.boldItalic, size: size)
    }
    
    class func extraBold(_ size: CGFloat) -> UIFont {
        getFont(.extraBold, size: size)
    }
    
    class func extraBoldItalic(_ size: CGFloat) -> UIFont {
        getFont(.extraBoldItalic, size: size)
    }
    
    class func extraLight(_ size: CGFloat) -> UIFont {
        getFont(.extraLight, size: size)
    }
}
