//
//  UIWindow.swift
//  TraccarClient
//
//  Created by George Makhoul on 29/10/2023.
//  Copyright © 2023 Traccar. All rights reserved.
//

import UIKit

func topViewController() -> UIViewController? {
    UIApplication.shared.windows.filter { $0.isKeyWindow }.first?.visibleViewController
}

extension UIWindow {
    var visibleViewController: UIViewController? {
        return UIWindow.visibleViewController(viewController: self.rootViewController)
    }
    
    static func visibleViewController(viewController: UIViewController?) -> UIViewController? {
        if let navigation = viewController as? UINavigationController {
            return UIWindow.visibleViewController(viewController: navigation.visibleViewController)
        } else if let tabBar = viewController as? UITabBarController {
            return UIWindow.visibleViewController(viewController: tabBar.selectedViewController)
        } else {
            if let presented = viewController?.presentedViewController {
                return UIWindow.visibleViewController(viewController: presented)
            } else {
                return viewController
            }
        }
    }
}
